# First setup of the model
#remove(list=ls()) # Remove all variables from memory
# install.packages("simecol")
require("simecol")
library('simecol')
# packages
#install.packages('tcltk')
library(tcltk)  
#install.packages('tictoc')
library(tictoc)

tic() # to determine time consumption 
# parameters 
resolution=0.1 # size of each cell, unit => m
length1=100 # long-shore lengh of the landscpae, unit => m
length2=100 # cross-shore length of the landscape, unit => m

n1=length1/resolution # number of rows
n2=length2/resolution # number of coloumns 

SDC= c(0, 0.1, 1) # Seedling abundance coefficient 

Tau=seq(from=0.01, to=0.07, by=0.0025) # TIme-averaged wave induced bed shear stress

Ks_max = 25/resolution #  Maximum value of the half satuation constant for seedling density decay with distance 
step_max= 2/resolution # maximum value of the clonal growth rate

Simulations=50 # Replicates for each scenario 
Store=matrix(nrow=length(SDC)*length(Tau)*Simulations, ncol=5, data=0) # for storing the output data 
Store=data.frame(Store) # make it a data frame
colnames(Store)=c('SDC','Tau','Simulations','Num.Year','Cover')
Count=0 # for counting the number of runs
EndTime=140 # if seedling establishment is not possible at all, the simulations stops when the time reaches this value, to avoid endless loop
            # This value is slightly longer than the time it needs to recover the landscape by only clonal growth


#----------------------------function of seedling estalbishment------------------------------
SeedlingEstablishment = function(landscape, PSD, Ks)
{       
        n1=dim(landscape)[1]
        n2=dim(landscape)[2]
        Closed=which(colSums(landscape==2)>=0.9*n1) # which rows are (almost) closed
        border = Closed[length(Closed)]  # find the border of the closed vegetation
        border = max(0, border) # if no border, then make sure border = 0
        
        if (border==n2){
                landscape=matrix(nrow=n1,ncol=n2, data=0)
        }
        else {
                distance=1:(n2-border)             # distance to the marsh edge 
                SD=PSD*Ks/(Ks+distance);              # Seedling density at a given distance
                Seedlingnum=round(n1*SD)     # number of seedling cells at a given distance (x coordinate)
                
                
                for (i in (border+1):n2)           # loop for each distance 
                {
                        sn=min(n1, Seedlingnum[i-border])       # Number of seedlings at that distance 
                        Seed_location=sample(n1,sn)     # Select the row numbers for the seedlings, from the randomized x coordinates 
                        landscape[Seed_location,i]=1     # Change the value to 1 for the cells that occupied by seedlings. 
                } 
        }
        return (landscape==1)*1              # return the new landscape 
        
}
# Turn on the progress bar
pb <- tkProgressBar("Progress","Completed %", 0, 100)  

for(i in 1:length(SDC)) {
        
        for(j in 1:length(Tau)) {
                
                SP=exp(-102.6*Tau[j]) #  Seed persistence as a funcion of wave-induced bed shear stress 
                PSD=SP*SDC[i] # peak seedling density (Averaged number of seedlings per cell
                     
                for (Si in 1:Simulations) {
                        
                        R=matrix(nrow=n1, ncol=n2, data=runif(n1*n2)) # a field of random values betweeon 0 and 1
                        Cells=matrix(nrow=n1,ncol=n2, data=0)       # generate a landscape 
                        Year=0
                        Closed=which(colSums(Cells==2)>=0.9*n1) 
                        border = Closed[length(Closed)] 
                        Cover=sum(Cells)/(n1*n2*2)
                        Clonalgrowth=NULL
                        
                        while (Cover < 0.9 & Year<EndTime){  
                                
                                Ks=sample(10:Ks_max,1) 
                                
                                if (sample(c(0,1),1)) # ramdomly determine whethere it is a bad year
                                { 
                                        Seedlings = SeedlingEstablishment(Cells,PSD, Ks)     
                                }
                                else {
                                        Seedlings = matrix(nrow=n1,ncol=n2, data=0)
                                }
                                
                                Year=Year+1
                                step=sample(1:step_max,1)
                                Clonalgrowth[Year]=step
                                XY=matrix(nrow=step*2+1, ncol=step*2+1, data=seq(-step,step),(step*2)+1)
                                z=sqrt(XY^2+t(XY)^2)<=step; z[step+1,step+1]=0 # Create a circular neighbourhood
                                GrowthNeighbourhood=z
                                
                                ## For each cell, calculate the number of vegetated neighbours
                                Sumveg=matrix(nrow=n1,ncol=n2, neighbours(x=Cells,state=2,wdist=GrowthNeighbourhood))
                                C=Sumveg/4 # clonal growth capacity. 
                                # Transition probability from a empty cell to a vegetated cell.
                                #If there are 4 or more vegetated cells, 
                                # in the neighbourhood, the clonal growth probability will be 1
                                
                                Growth_Clone=(Seedlings==0)&(Cells==0)&(R<=C)
                                Growth_Clone=2*Growth_Clone
                                
                                # Vegetation growth due to seedlings; Each seedling cell (1) becomes a fully vegetated cell (2) 
                                Seedlings=(Seedlings==1)&(Cells==0); #Exclude the overlap of seedling cells and vegetated cells 
                                Growth_Seedling = Seedlings*2;
                                
                                # The new cell states 
                                Cells=Cells+Growth_Clone+Growth_Seedling 
                                Cover=sum(Cells)/(n1*n2*2)
                                
                        }
                        
                        Count=Count+1
                        k=dim(Store)[1]
                        Store$clonegrowth[Count]=mean(Clonalgrowth)
                        Store$Cover[Count]=Cover
                        # Display the progress
                        info<- sprintf("Completed %d%%", round(Count*100/k))  
                        setTkProgressBar(pb, Count*100/k, sprintf("Progress (%s)", info),info) 
                        
                        Store$Num.Year[Count]=Year
                        Store$Simulations[Count]=Si
                        Store$Tau[Count]=Tau[j]
                        Store$SDC[Count]=SDC[i]
                        
                }
         } 
        
}  

#close the progress bar  
close(pb)  

toc()